from panoramaTool.panorama_api.api_call import APICall
from panoramaTool.panorama_api.post_rules_manager import PostRulesManager
from panoramaTool.panorama_api.service_manager import ServiceManager

__version__ = '0.1.11'



import argparse
import json
import os
from pprint import pprint

from flask import Flask, render_template, request, make_response, redirect, url_for
from dotenv import load_dotenv

from panoramaTool import PostRulesManager
from panoramaTool.logic.business_logic import BusinessLogic
from panoramaTool.logic.csv_manager import CSVManager
from panoramaTool.panorama_api.api_call import APICall

load_dotenv()

app = Flask(__name__)
app.secret_key = 'this_is_a_super_unsecure_secret_key_here'
template_path = os.path.join(os.path.dirname(__file__), 'templates')
app.template_folder = template_path
app.static_folder = os.path.join(os.path.dirname(__file__), 'static')


@app.route('/', methods=['GET'])
def index():
    if BusinessLogic.check_for_valid_session(request):
        print("Valid Session")
        response = make_response(redirect(url_for('addresses')))
        return response
    response = make_response(redirect(url_for('login')))
    print("Not a valid session. Redirect to login!")
    return response


@app.route('/login', methods=['GET', 'POST'])
def login():
    response = make_response(redirect(url_for('addresses')))
    if request.method == 'GET' and not BusinessLogic.check_for_valid_session(request):
        print("Not a valid session. Login.")
        response = render_template('login.html')
    elif request.method == 'POST':
        panorama_url = request.form['url']
        username = request.form['username']
        password = request.form['password']
        api_key = APICall.get_api_key(user=username, password=password, panorama_url=panorama_url)
        if api_key == "Invalid Credentials":
            response = make_response(redirect(url_for('index')))
            return response
        response.set_cookie('panorama_url', panorama_url)
        response.set_cookie('api_key', api_key)
    return response


@app.route('/addresses', methods=['GET', 'POST'])
def addresses():
    if request.method == 'GET' and BusinessLogic.check_for_valid_session(request):
        return make_response(render_template('addresses.html'))
    elif request.method == 'POST':
        save_path = CSVManager.save_csv(request.files['csv_file'])
        csv_data = CSVManager.read_csv(save_path)
        BusinessLogic.make_concurrent_calls(
            request=request,
            function=BusinessLogic.create_address,
            csv=csv_data
        )
        return make_response(redirect(url_for('addresses')))
    return make_response(redirect(url_for('index')))


@app.route('/services', methods=['GET', 'POST'])
def services():
    if request.method == 'GET' and BusinessLogic.check_for_valid_session(request):
        return make_response(render_template('services.html'))
    elif request.method == 'POST':
        save_path = CSVManager.save_csv(request.files['csv_file'])
        csv_data = CSVManager.read_csv(save_path)
        BusinessLogic.make_concurrent_calls(
            request=request,
            function=BusinessLogic.create_services,
            csv=csv_data
        )
        return make_response(redirect(url_for('services')))
    return make_response(redirect(url_for('index')))


@app.route('/address_groups', methods=['GET', 'POST'])
def address_groups():
    if request.method == 'GET' and BusinessLogic.check_for_valid_session(request):
        return make_response(render_template('address_groups.html'))
    elif request.method == 'POST':
        save_path = CSVManager.save_csv(request.files['csv_file'])
        csv_data = CSVManager.read_csv(save_path)
        BusinessLogic.make_concurrent_calls(
            request=request,
            function=BusinessLogic.create_address_groups,
            csv=csv_data
        )
        return make_response(redirect(url_for('address_groups')))
    return make_response(redirect(url_for('index')))


@app.route('/service_groups', methods=['GET', 'POST'])
def service_groups():
    if request.method == 'GET' and BusinessLogic.check_for_valid_session(request):
        return make_response(render_template('service_groups.html'))
    elif request.method == 'POST':
        save_path = CSVManager.save_csv(request.files['csv_file'])
        csv_data = CSVManager.read_csv(save_path)
        BusinessLogic.make_concurrent_calls(
            request=request,
            function=BusinessLogic.create_service_groups,
            csv=csv_data
        )
        return make_response(redirect(url_for('service_groups')))
    return make_response(redirect(url_for('index')))


@app.route('/security_rules', methods=['GET', 'POST'])
def security_rules():
    if request.method == 'GET' and BusinessLogic.check_for_valid_session(request):
        faulty_str = request.cookies.get('faulty_requests')
        if faulty_str is not None:
            faulty = json.loads(faulty_str)
            try:
                print(f"faulty: {faulty[0][0]['NAME']}")
                print(f"faulty: {faulty[0][1]['details'][0]['causes'][0]['description']}")
            except:
                print(f"faulty: {faulty}")
            return make_response(render_template('security_rules.html',
                                 faulty_response=faulty))
        return make_response(render_template('security_rules.html'))
    elif request.method == 'POST':
        save_path = CSVManager.save_csv(request.files['csv_file'])
        csv_data = CSVManager.read_csv(save_path)
        faulty_requests = BusinessLogic.make_concurrent_calls(
            request=request,
            function=BusinessLogic.create_security_post_rules,
            csv=csv_data
        )
        response = make_response(redirect(url_for('security_rules')))
        response.set_cookie('faulty_requests', json.dumps(faulty_requests))
        return response
    return make_response(redirect(url_for('index')))


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--port', type=int, default=5000, help='Port number')
    args = parser.parse_args()
    app.run(port=args.port)



import os
import csv

class CSVManager:
    @staticmethod
    def save_csv(csv_file):
        csv_folder = os.path.join(os.path.dirname(__file__), 'csv')
        if not os.path.exists(csv_folder):
            os.makedirs(csv_folder)
        save_path = os.path.join(csv_folder, 'upload.csv')
        if os.path.exists(save_path):
            os.remove(save_path)
        csv_file.save(save_path)
        return save_path

    @staticmethod
    def read_csv(file_path):
        csv_data = []
        with open(file_path, 'r', encoding='utf-8-sig') as file:
            reader = csv.DictReader(file, delimiter=';')
            for row in reader:
                csv_data.append(row)
        return csv_data



import json

import requests


class PostRulesManager:
    def __init__(self, panorama_url, api_key, verify=True):
        self.panorama_url = panorama_url
        self.api_key = api_key
        self.verify = verify
        self.url = f"{self.panorama_url}/restapi/v10.2/Policies/SecurityPostRules"
        self.headers = {'Content-Type': 'application/json',
                        'X-PAN-KEY': f"{self.api_key}"}

    def list_post_rules(self):
        params = {'location': 'shared', 'device-group': 'TestGroup'}
        response = requests.get(url=self.url, headers=self.headers, params=params, verify=self.verify)
        post_rules = json.loads(response.content.decode('utf-8'))
        return post_rules

    def create_post_rules(self, device_group, name, action, description, from_zone, source_group,
                          to_zone, destination_group, service, profile_type, profile):
        location = "shared" if device_group == 'shared' else "device-group"
        params = {'name': name, 'location': location, 'device-group': device_group}
        payload = self._get_post_rules_payload(device_group, name, action, description, from_zone, source_group,
                                               to_zone, destination_group, service, profile_type, profile)
        response = requests.post(url=self.url, headers=self.headers, params=params, data=json.dumps(payload),
                                 verify=self.verify)
        return json.loads(response.content.decode('utf-8'))

    def _get_post_rules_payload(self, device_group, name, action, description, from_zone, source_group,
                                to_zone, destination_group, service, profile_type, profile):
        security_policy_payload = {
            "entry": [
                {
                    "@location": device_group,
                    "@name": name,
                    "description": description,
                    "@vsys": "vsys1",
                    "action": action,
                    "log-setting": "Default",
                    "from": {
                        "member": [
                            from_zone.split('\n')
                        ]
                    },
                    "to": {
                        "member": [
                            to_zone.split('\n')
                        ]
                    },
                    "source": {
                        "member": [
                            source_group.split('\n')
                        ]
                    },
                    "destination": {
                        "member": [
                            destination_group.split('\n')
                        ]
                    },
                    "service": {
                        "member": [
                            service.split('\n')
                        ]
                    },
                    "application": {
                        "member": [
                            "any"
                        ]
                    },
                    "tag": {
                        "member": [
                            "to_be_reviewed"
                        ]
                    },
                    'profile-setting': {
                        profile_type: {
                            'member': [
                                profile
                            ]
                        }
                    }
                }
            ]
        }
        return security_policy_payload



import json

import requests


class AddressManager:
    def __init__(self, panorama_url, api_key, verify=True):
        self.panorama_url = panorama_url
        self.api_key = api_key
        self.verify = verify
        self.url = f"{self.panorama_url}/restapi/v10.2/Objects/Addresses"
        self.headers = {'Content-Type': 'application/json',
                        'X-PAN-KEY': f"{self.api_key}"}

    def list_addresses(self):
        params = {'location': 'shared', 'device-group': 'TestGroup'}
        response = requests.get(url=self.url, headers=self.headers, params=params, verify=self.verify)
        addresses = json.loads(response.content.decode('utf-8'))
        return addresses

    def create_address(self, name, ip, cidr, description):
        params = {'name': name, 'location': 'shared', 'device-group': 'TestGroup'}
        payload = self._get_address_payload(name, ip, cidr, description)
        response = requests.post(url=self.url, headers=self.headers, params=params, data=json.dumps(payload),
                                 verify=self.verify)
        return json.loads(response.content.decode('utf-8'))

    def _get_address_payload(self, name, ip, cidr, description):
        address_payload = {
            'entry': [
                {
                    '@name': name,
                    '@location': 'TestGroup',
                    'ip-netmask': f'{ip}/{cidr}',
                    'description': description
                }
            ]
        }
        return address_payload



import json

import requests


class ServiceGroupsManager:
    def __init__(self, panorama_url, api_key, verify=True):
        self.panorama_url = panorama_url
        self.api_key = api_key
        self.verify = verify
        self.url = f"{self.panorama_url}/restapi/v10.2/Objects/ServiceGroups"
        self.headers = {"Content-Type": "application/json",
                        'X-PAN-KEY': f"{self.api_key}"}

    def list_service_groups(self):
        params = {'location': 'shared', 'device-group': 'TestGroup'}
        response = requests.get(url=self.url, headers=self.headers, params=params, verify=self.verify)
        service_groups = json.loads(response.content.decode('utf-8'))
        return service_groups

    def create_service_group(self, device_group: str, name: str, members: str):
        location = "shared" if device_group == 'shared' else "device-group"
        params = {'name': name, 'location': location, 'device-group': device_group}
        payload = self._get_service_group_payload(device_group=device_group, name=name, members=members)
        response = requests.post(url=self.url, headers=self.headers, params=params, data=json.dumps(payload),
                                 verify=self.verify)
        service_group = json.loads(response.content.decode('utf-8'))
        return service_group

    def _get_service_group_payload(self, device_group: str, name: str, members: str):
        service_group_payload = {
            'entry': [
                {
                    '@name': name,
                    '@location': device_group,
                    'members':
                        {
                            'member': members.split('\n')
                        },
                }
            ]
        }
        print(service_group_payload)
        return service_group_payload



import concurrent.futures
from panoramaTool import APICall, PostRulesManager, ServiceManager
from panoramaTool.panorama_api.address_groups_manager import AddressGroupsManager
from panoramaTool.panorama_api.address_manager import AddressManager
from panoramaTool.panorama_api.service_groups_manager import ServiceGroupsManager


class BusinessLogic:
    def __init__(self, url, api_key):
        self.url = url
        self.api_key = api_key

    @staticmethod
    def check_for_valid_session(request) -> bool:
        url = request.cookies.get('panorama_url')
        api_key = request.cookies.get('api_key')
        if url is None or api_key is None:
            return False
        valid = APICall.check_valid_key(panorama_url=url, api_key=api_key)
        return valid

    @staticmethod
    def make_concurrent_calls(request, function, csv):
        url = request.cookies.get('panorama_url')
        api_key = request.cookies.get('api_key')
        device_group = request.form.get('device-group') or 'shared'
        response = []
        faulty_response = []

        with concurrent.futures.ThreadPoolExecutor() as executor:
            futures = []
            data = []
            for csv_data in csv:
                futures.append(executor.submit(function, url, api_key, csv_data, device_group))
                data.append(csv_data)

            concurrent.futures.wait(futures)

            for counter, future in enumerate(futures):
                if "success" in str(future.result()):
                    response.append(future.result())
                else:
                    print((data[counter], future.result()))
                    faulty_response.append((data[counter], future.result()))
        return faulty_response

    @staticmethod
    def create_address(url, api_key, csv_data, device_group):
        address_manager = AddressManager(panorama_url=url, api_key=api_key, verify=False)
        return address_manager.create_address(
            name=csv_data.get('NAME').replace(" ", ""),
            ip=csv_data.get('IP ADDRESS').replace(" ", ""),
            cidr=csv_data.get('CIDR').replace(" ", "") or '32',
            description=csv_data.get('DESCRIPTION').replace(" ", "") or ''
        )

    @staticmethod
    def create_services(url, api_key, csv_data, device_group):
        service_manager = ServiceManager(panorama_url=url, api_key=api_key, verify=False)
        return service_manager.create_service(
            protocol=csv_data.get('PROTOCOL').replace(" ", ""),
            port=csv_data.get('DESTINATION PORT').replace(" ", ""),
            name=csv_data.get('NAME').replace(" ", "")
        )

    @staticmethod
    def create_address_groups(url, api_key, csv_data, device_group):
        address_groups_manager = AddressGroupsManager(panorama_url=url, api_key=api_key, verify=False)
        return address_groups_manager.create_address_group(
            name=csv_data.get('NAME').replace(" ", ""),
            members=csv_data.get('MEMBERS').replace(" ", ""),
            description=csv_data.get('DESCRIPTION').replace(" ", "")
        )

    @staticmethod
    def create_service_groups(url, api_key, csv_data, device_group):
        service_groups_manager = ServiceGroupsManager(panorama_url=url, api_key=api_key, verify=False)
        return service_groups_manager.create_service_group(
            device_group=device_group,
            name=csv_data.get('NAME').replace(" ", ""),
            members=csv_data.get('MEMBERS').replace(" ", "")
        )

    @staticmethod
    def create_security_post_rules(url, api_key, csv_data, device_group):
        security_post_rules_manager = PostRulesManager(panorama_url=url, api_key=api_key, verify=False)
        return security_post_rules_manager.create_post_rules(
            device_group=device_group,
            name=csv_data.get('NAME').lstrip(),
            action=csv_data.get('ACTION').lstrip() or 'allow',
            description=csv_data.get('DESCRIPTION').lstrip() or '',
            from_zone=csv_data.get('FROM').lstrip() or 'any',
            source_group=csv_data.get('SOURCE').lstrip() or 'any',
            to_zone=csv_data.get('TO').lstrip() or 'any',
            destination_group=csv_data.get('DESTINATION').lstrip() or 'any',
            service=csv_data.get('SERVICE').lstrip() or 'application-default',
            profile_type=csv_data.get('PROFILE TYPE').lstrip() or 'group',
            profile=csv_data.get('PROFILE').lstrip() or 'Alert-Only'
        )
